% This is a file for replication of results in Section 8.3
clear all;
%% Generating (Y,D,Z,W) of size N
% Size and seed
N = 100000;
K=50; % Set the polynomial order
K=K+1;
rng(123)

% Parameters used in data generating process
p_Z = 0.5; % sets the probability of Z=1
p_W = 0.4; % sets the probability of W=1
p_X = 0.6; % sets the probability of X=1

nu_z = 0.3; % coefficients determining D
nu_x = 0.3;
rho = 0.3; % set X to be endogenous, if X is exogeneous, set to 0

cop_mat = copularnd('Gaussian',rho,N);
u_x = cop_mat(:,1);
e = cop_mat(:,2);
u = rand(N,1); % latent variable determining both D and Y (via MTR functions)

% The MTR functions
m_0_0(:,1) = 0.01*bern(4,0,u) + 0.02*bern(4,1,u) + 0.02*bern(4,2,u) + 0.02*bern(4,3,u) + 0.02*bern(4,4,u);
m_1_0(:,1) = 0.03*bern(4,0,u) + 0.06*bern(4,1,u) + 0.09*bern(4,2,u) + 0.12*bern(4,3,u) + 0.12*bern(4,4,u);
m_0_1(:,1) = 0.05*bern(4,0,u) + 0.54*bern(4,1,u) + 0.73*bern(4,2,u) + 0.84*bern(4,3,u) + 0.86*bern(4,4,u);
m_1_1(:,1) = 0.94*bern(4,0,u) + 0.95*bern(4,1,u) + 0.96*bern(4,2,u) + 0.96*bern(4,3,u) + 0.96*bern(4,4,u);

m_0_0(:,2) = 0.01*bern(4,0,u) + 0.02*bern(4,1,u) + 0.03*bern(4,2,u) + 0.04*bern(4,3,u) + 0.04*bern(4,4,u);
m_1_0(:,2) = 0.01*bern(4,0,u) + 0.05*bern(4,1,u) + 0.09*bern(4,2,u) + 0.13*bern(4,3,u) + 0.13*bern(4,4,u);
m_0_1(:,2) = 0.26*bern(4,0,u) + 0.61*bern(4,1,u) + 0.84*bern(4,2,u) + 0.93*bern(4,3,u) + 0.94*bern(4,4,u);
m_1_1(:,2) = 0.95*bern(4,0,u) + 0.96*bern(4,1,u) + 0.97*bern(4,2,u) + 0.98*bern(4,3,u) + 0.99*bern(4,4,u);

% Data
Z = p_Z >= rand(N,1);
W = p_W >= rand(N,1);
X = p_X >= u_x;

D = (0.1+nu_z*Z+nu_x*X >= u);
Y = D.*(W.*(X.*(m_1_1(:,2)>e)+(1-X).*(m_1_1(:,1))>e)...
    +(1-W).*(X.*(m_1_0(:,2)>e)+(1-X).*(m_1_0(:,1)>e)))...
    +(1-D).*(W.*(X.*(m_0_1(:,2)>e)+(1-X).*(m_0_1(:,1)>e))...
    +(1-W).*(X.*(m_0_0(:,2)>e)+(1-X).*(m_0_0(:,1)>e))); 

ATE = mean(W.*(X.*(m_1_1(:,2)>e)+(1-X).*(m_1_1(:,1)>e))...
    +(1-W).*(X.*(m_1_0(:,2)>e)+(1-X).*(m_1_0(:,1)>e)))...
    -mean(W.*(X.*(m_0_1(:,2)>e)+(1-X).*(m_0_1(:,1)>e))...
    +(1-W).*(X.*(m_0_0(:,2)>e)+(1-X).*(m_0_0(:,1)>e)));

%% Basis Function
% generate basis functions based on bernstein polynomials
b = cell(K,1);
for i=1:K
    b{i} = @(u)bern(K-1,i-1,u);
end

%% Coefficient matrix A (for ATE)
% Determining support of the variables
supp_d = unique(D); % determining the support of D, W, X
supp_x = unique(X);
supp_w = unique(W);
supp_z = unique(Z);

dim_d = size(supp_d,1); % dimension of D, W, X
dim_x = size(supp_x,1);
dim_w = size(supp_w,1);
dim_z = size(supp_z,1);

dim_s = 2^(dim_d*dim_w); % number of maps
s = (1:dim_s)';
Bi_s = de2bi(s-1);    % value of y(d,w) for each (d,w)

% Sompe proabilities used in calculating A
% joint probablity of W and X in data
p_wx = zeros(2,dim_x);
for j = 1:dim_x
    for i=1:dim_w
        p_wx(i,j) = sum(W==supp_w(i)&X==supp_x(j))/N;
    end
end
% probability of W conditional on X
p_w_x = zeros(2,dim_x);
for j = 1:dim_x
    for i=1:dim_w
        p_w_x(i,j) = sum(W==supp_w(i)&X==supp_x(j))/sum(X==supp_x(j));
    end
end

% Create coefficient matrix A
A_0 = zeros(K,dim_s,dim_x);
A_1 = zeros(K,dim_s,dim_x);

gam0_ATE = zeros(K,1);
gam1_ATE = zeros(K,1);

for i=1:K
    syms u
    gam0_ATE(i) = -int(b{i},u,0,1);
    gam1_ATE(i) = int(b{i},u,0,1);
end

for i=1:dim_x
    for j = 1:dim_s
        for m=1:dim_w
            if Bi_s(j,m) == 1  % in Bi_s, find the column for y(d,w) and choose s (decimal value j) s.t. y(0,w) = 1
                A_0(:,j,i) = p_wx(m,i)+A_0(:,j,i); % sum across w s.t y(0,w) = 1
            end
            if Bi_s(j,m+2) == 1  % in Bi_s, find the column for y(d,w) and choose s (decimal value j) s.t. y(1,w) = 1
                A_1(:,j,i) = p_wx(m,i)+A_1(:,j,i); % sum across w s.t y(1,w) = 1
            end
        end
        A_0(:,j,i) = A_0(:,j,i).*gam0_ATE;
        A_1(:,j,i) = A_1(:,j,i).*gam1_ATE;
    end
end

A = reshape(A_0,1,[])+reshape(A_1,1,[]);

%% Defining the restriction distributions p=P(D,Y=1|Z,W,X)
dim_p = dim_z*dim_d*dim_x*dim_w;    % number of possibility in (Z,D,Y,W,X) 
Bi = de2bi(0:dim_p-1);      % the order of column: (Z,D,W,X); each ROW of this matrix correspond to each decimal number
p = zeros(dim_p,1);
for i = 1:dim_p
    summand1 = (D==Bi(i,2)).*(W==Bi(i,3)).*(Z==Bi(i,1)).*(Y==1).*(X==Bi(i,4));
    summand2 = (Z==Bi(i,1)).*(W==Bi(i,3)).*(X==Bi(i,4));
    p(i) = sum(summand1)/sum(summand2); % This is the data restriction
end

p_z = zeros(dim_z,dim_w,dim_x); % the propensity score
for i=1:dim_z
    for j=1:dim_w
        for m=1:dim_x
            p_z(i,j,m) = sum((D==1).*(Z==supp_z(i)).*(W==supp_w(j)).*(X==supp_x(m)))...
                /sum((Z==supp_z(i)).*(W==supp_w(j)).*(X==supp_x(m)));
        end
    end
end

%% Defining coefficient matrices LATE for LATEs(NT, AT, complier)
gam0_LATE = zeros(K,dim_x,dim_w,3);
gam1_LATE = zeros(K,dim_x,dim_w,3);

for w=1:dim_w
    for i=1:K
        for j=1:dim_x
            for m=1:3
            syms u
            gam0_LATE(i,j,w,m)=-int(b{i},u,(m==2)*p_z(1,w,j)+(m==3)*p_z(2,w,j),(m==1)*p_z(1,w,j)+(m==2)*p_z(2,w,j)+(m==3))...
                /((m==1)*p_z(1,w,j)+(m==2)*p_z(2,w,j)+(m==3)-(m==2)*p_z(1,w,j)-(m==3)*p_z(2,w,j));
            gam1_LATE(i,j,w,m)= int(b{i},u,(m==2)*p_z(1,w,j)+(m==3)*p_z(2,w,j),(m==1)*p_z(1,w,j)+(m==2)*p_z(2,w,j)+(m==3))...
                /((m==1)*p_z(1,w,j)+(m==2)*p_z(2,w,j)+(m==3)-(m==2)*p_z(1,w,j)-(m==3)*p_z(2,w,j));
            end
        end
    end
end


LATE = zeros(2,K*dim_s*dim_x,3);
for m=1:3
    L_00_0 = zeros(K,dim_s,dim_x);
    L_01_0 = zeros(K,dim_s,dim_x);
    L_10_0 = zeros(K,dim_s,dim_x);
    L_11_0 = zeros(K,dim_s,dim_x);
    L_00_1 = zeros(K,dim_s,dim_x);
    L_01_1 = zeros(K,dim_s,dim_x);
    L_10_1 = zeros(K,dim_s,dim_x);
    L_11_1 = zeros(K,dim_s,dim_x);
    for i = 1:dim_x
        for j = 1:dim_s
            for w=1:dim_w
                if Bi_s(j,1) == 1  % in Bi_s, for each x, find the column for y(d,w) and choose s (decimal value j) s.t. y(d,w) = 1
                    L_00_0(:,j,i) = (-1*i+2);
                    L_00_1(:,j,i) = (i-1);
                end
                if Bi_s(j,2) == 1  % in Bi_s, for each x, find the column for y(d,w) and choose s (decimal value j) s.t. y(d,w) = 1
                    L_01_0(:,j,i) = (-1*i+2);
                    L_01_1(:,j,i) = (i-1);
                end
                if Bi_s(j,3) == 1  % in Bi_s, for each x, find the column for y(d,w) and choose s (decimal value j) s.t. y(d,w) = 1
                    L_10_0(:,j,i) = (-1*i+2);
                    L_10_1(:,j,i) = (i-1);
                end
                if Bi_s(j,4) == 1  % in Bi_s, for each x, find the column for y(d,w) and choose s (decimal value j) s.t. y(d,w) = 1
                    L_11_0(:,j,i) = (-1*i+2);
                    L_11_1(:,j,i) = (i-1);
                end
                L_00_0(:,j,i) = L_00_0(:,j,i).*gam0_LATE(:,i,1,m);
                L_01_0(:,j,i) = L_01_0(:,j,i).*gam0_LATE(:,i,2,m);
                L_10_0(:,j,i) = L_10_0(:,j,i).*gam1_LATE(:,i,1,m);
                L_11_0(:,j,i) = L_11_0(:,j,i).*gam1_LATE(:,i,2,m);
                L_00_1(:,j,i) = L_00_1(:,j,i).*gam0_LATE(:,i,1,m);
                L_01_1(:,j,i) = L_01_1(:,j,i).*gam0_LATE(:,i,2,m);
                L_10_1(:,j,i) = L_10_1(:,j,i).*gam1_LATE(:,i,1,m);
                L_11_1(:,j,i) = L_11_1(:,j,i).*gam1_LATE(:,i,2,m);
            end
        end
    end
    LATE(1,:,m)=p_w_x(1,1).*reshape(L_00_0,1,[])+p_w_x(2,1).*reshape(L_01_0,1,[])...
        +p_w_x(1,1).*reshape(L_10_0,1,[])+p_w_x(2,1).*reshape(L_11_0,1,[]);
    LATE(2,:,m)=p_w_x(1,2).*reshape(L_00_1,1,[])+p_w_x(2,2).*reshape(L_01_1,1,[])...
        +p_w_x(1,2).*reshape(L_10_1,1,[])+p_w_x(2,2).*reshape(L_11_1,1,[]);
end

%% Defining B: the coefficent matrix for data restriction
% the left hand side coefficients
B = [];
for l = 1:dim_p
    
    z = Bi(l,1); % assign values for later calculation to match with corresponding row in p
    d = Bi(l,2); 
    y = 1;
    w = Bi(l,3);
    x = Bi(l,4);
    
    delta = zeros(1,K);
    for m=1:K
        if d == 1
            delta(m) = int(b{m},u,0,p_z(z+1,w+1,x+1));
        elseif d == 0
            delta(m) = int(b{m},u,p_z(z+1,w+1,x+1),1);
        end
    end
    
    B0 = zeros(K,dim_s,dim_x);
    
    for j = 1:dim_s
        if Bi_s(j,2*d+w+1) == y  % in Bi_s, find the column for y(d,w) and choose s (decimal value j) s.t. y(d,w) = y
            B0(:,j,x+1) = 1;
        end
        B0(:,j,x+1) = delta'.*B0(:,j,x+1);
    end
    B0 = reshape(B0,1,[]);
    B(l,:) = B0;
end

%% Inequality restrictions on conditional probabilities
% without rank similarity assumption 
% (with rank similarity assumption, there will be less mapping types)
dim = size(B,2); % dimension of thetas
zer = zeros(dim,1); % the restriction on the rhs
one = zeros(K,dim,dim_x); % coefficient matrix on the lhs

for i=1:K
    for j=1:dim
        if mod(j-i,K)==0 && j<=dim_s*K
            one(i,j,1) = 1;
        elseif mod(j-i,K)==0 && j>dim_s*K
            one(i,j,2) = 1;
        end
    end
end

one = [one(:,:,1);one(:,:,2)];

%% Defining additional assumptions
% Monotonicity (Assumption M)
M00 = zeros(dim_x*(K-1),dim);
M01 = zeros(dim_x*(K-1),dim);
M10 = zeros(dim_x*(K-1),dim);
M11 = zeros(dim_x*(K-1),dim);

for x = 1:dim_x
    if x==1 % Monotonicity for X==0 
        for i = 1:K-1
            M_0_0 = zeros(K,dim_s);
            M_1_0 = zeros(K,dim_s);
            M_0_1 = zeros(K,dim_s);
            M_1_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % finding those types with y(0,0)=1
                    M_0_0(i,j) = -1;
                    M_0_0(i+1,j) = 1; % theta_k+1-theta_k>0
                end
                if Bi_s(j,2) == 1 % finding those types with y(0,1)=1
                    M_0_1(i,j) = -1;
                    M_0_1(i+1,j) = 1;
                end
                if Bi_s(j,3) == 1 % finding those types with y(1,0)=1
                    M_1_0(i,j) = -1;
                    M_1_0(i+1,j) = 1;
                end
                if Bi_s(j,4) == 1 % finding those types with y(1,1)=1
                    M_1_1(i,j) = -1;
                    M_1_1(i+1,j) = 1;
                end
            end
            M_0_0 = reshape(M_0_0,1,[]); % convert into a row
            M_0_1 = reshape(M_0_1,1,[]);
            M_1_0 = reshape(M_1_0,1,[]);
            M_1_1 = reshape(M_1_1,1,[]);
            M00(i,1:dim/2) = M_0_0; % fill the coefficient matrix for x=0
            M01(i,1:dim/2) = M_0_1;
            M10(i,1:dim/2) = M_1_0;
            M11(i,1:dim/2) = M_1_1;
        end
    else
        for i = K:2*(K-1) % Monotonicity for X==1
            M_0_0 = zeros(K,dim_s);
            M_1_0 = zeros(K,dim_s);
            M_0_1 = zeros(K,dim_s);
            M_1_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 
                    M_0_0(i-K+1,j) = -1;
                    M_0_0(i-K+2,j) = 1;
                end
                if Bi_s(j,2) == 1 
                    M_0_1(i-K+1,j) = -1;
                    M_0_1(i-K+2,j) = 1;
                end
                if Bi_s(j,3) == 1 
                    M_1_0(i-K+1,j) = -1;
                    M_1_0(i-K+2,j) = 1;
                end
                if Bi_s(j,4) == 1 
                    M_1_1(i-K+1,j) = -1;
                    M_1_1(i-K+2,j) = 1;
                end
            end
            M_0_0 = reshape(M_0_0,1,[]);
            M_0_1 = reshape(M_0_1,1,[]);
            M_1_0 = reshape(M_1_0,1,[]);
            M_1_1 = reshape(M_1_1,1,[]);
            M00(i,dim/2+1:dim) = M_0_0; % fill the coefficient matrix for x=1
            M01(i,dim/2+1:dim) = M_0_1;
            M10(i,dim/2+1:dim) = M_1_0;
            M11(i,dim/2+1:dim) = M_1_1;
        end
    end
end

M = [M00;M01;M10;M11];

% Concavity (Assumption C)
C00 = zeros(dim_x*(K-2),dim);
C01 = zeros(dim_x*(K-2),dim);
C10 = zeros(dim_x*(K-2),dim);
C11 = zeros(dim_x*(K-2),dim);
for x = 1:dim_x
    if x==1
        for i = 1:K-2
            C_0_0 = zeros(K,dim_s);
            C_0_1 = zeros(K,dim_s);
            C_1_0 = zeros(K,dim_s);
            C_1_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % finding those types with y(0,0)=1
                    C_0_0(i,j) = 1; % theta_k-2*theta_{k+1}+theta_{k+2}<0
                    C_0_0(i+1,j) = -2;
                    C_0_0(i+2,j) = 1;
                end
                if Bi_s(j,2) == 1 % finding those types with y(0,1)=1
                    C_0_1(i,j) = 1;
                    C_0_1(i+1,j) = -2;
                    C_0_1(i+2,j) = 1;
                end
                if Bi_s(j,3) == 1 % finding those types with y(1,0)=1
                    C_1_0(i,j) = 1;
                    C_1_0(i+1,j) = -2;
                    C_1_0(i+2,j) = 1;
                end
                if Bi_s(j,4) == 1 % finding those types with y(1,1)=1
                    C_1_1(i,j) = 1;
                    C_1_1(i+1,j) = -2;
                    C_1_1(i+2,j) = 1;
                end
            end
            C_0_0 = reshape(C_0_0,1,[]); % convert into a row
            C_0_1 = reshape(C_0_1,1,[]);
            C_1_0 = reshape(C_1_0,1,[]);
            C_1_1 = reshape(C_1_1,1,[]);
            C00(i,1:dim/2) = C_0_0; % fill the coefficient matrix for x=0
            C01(i,1:dim/2) = C_0_1;
            C10(i,1:dim/2) = C_1_0;
            C11(i,1:dim/2) = C_1_1;
        end
    else
        for i = K-1:dim_x*(K-2)
            C_0_0 = zeros(K,dim_s);
            C_0_1 = zeros(K,dim_s);
            C_1_0 = zeros(K,dim_s);
            C_1_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % finding those types with y(0,0)=1
                    C_0_0(i-K+2,j) = 1;
                    C_0_0(i-K+3,j) = -2;
                    C_0_0(i-K+4,j) = 1;
                end
                if Bi_s(j,2) == 1 % finding those types with y(0,1)=1
                    C_0_1(i-K+2,j) = 1;
                    C_0_1(i-K+3,j) = -2;
                    C_0_1(i-K+4,j) = 1;
                end
                if Bi_s(j,3) == 1 % finding those types with y(1,0)=1
                    C_1_0(i-K+2,j) = 1;
                    C_1_0(i-K+3,j) = -2;
                    C_1_0(i-K+4,j) = 1;
                end
                if Bi_s(j,4) == 1 % finding those types with y(1,1)=1
                    C_1_1(i-K+2,j) = 1;
                    C_1_1(i-K+3,j) = -2;
                    C_1_1(i-K+4,j) = 1;
                end
            end
            C_0_0 = reshape(C_0_0,1,[]);
            C_0_1 = reshape(C_0_1,1,[]);
            C_1_0 = reshape(C_1_0,1,[]);
            C_1_1 = reshape(C_1_1,1,[]);
            C00(i,dim/2+1:dim) = C_0_0; % fill the coefficient matrix for x=1
            C01(i,dim/2+1:dim) = C_0_1;
            C10(i,dim/2+1:dim) = C_1_0;
            C11(i,dim/2+1:dim) = C_1_1;
        end
    end
end

C = [C00;C01;C10;C11];

%% Determining the RSW direction (U*)
% Matrices required under assumption U*
% choose direction for x=0
[A_R1_old, B_R1_old, H1_old, one_R1_old, dim_R1_old, zer_R1_old] = ChooseRSW(A(1:length(A)/2), B((1:length(p)/2),1:length(A)/2), p(1:length(p)/2), K, dim_s,dim_w,Bi_s);
% choose direction for x=1
[A_R2_old, B_R2_old, H2_old, one_R2_old, dim_R2_old, zer_R2_old] = ChooseRSW(A(length(A)/2+1:length(A)), B((length(p)/2+1:length(p)),length(A)/2+1:length(A)), p(length(p)/2+1:length(p)), K, dim_s,dim_w,Bi_s);

% create the target coefficient matrix under Assumption U*
H_old = [H1_old,H2_old]; % direction matrix
A_R_old = [A_R1_old,A_R2_old]; % ATE target coefficient
B_R_old = [B_R1_old,zeros(size(B_R2_old));zeros(size(B_R1_old)),B_R2_old]; % data restriction coefficient
one_R_old = [one_R1_old,zeros(size(one_R2_old));zeros(size(one_R1_old)),one_R2_old]; % probability restriction coefficient
zer_R_old = [zer_R1_old;zer_R2_old]; 
dim_R_old = dim_R1_old+dim_R2_old; 

% use direction matrix to create new LATE coefficient
for i=1:3
    for j=1:dim_x
        temp = LATE(j,:,i);
        LATE_R_old(j,:,i)= temp(H_old~=-1);
    end
end

% use direction matrix to create new monotonicity and concavity coefficient
% matrices
for i=1:size(M,1)
    temp = M(i,:);
    M_R_old(i,:) = temp(H_old~=-1);
end

for i=1:size(C,1)
    temp = C(i,:);
    C_R_old(i,:) = temp(H_old~=-1);
end

%% Determining the assumption U direction
% matrices required under assumption U
% choose direction and create ATE target coefficient for x=0 and x=1
% separately
[A_R1, B_R1, H1, one_R1, dim_R1, zer_R1] = ChooseRSW_new(A(1:length(A)/2), B((1:length(p)/2),1:length(A)/2), p(1:length(p)/2), K, dim_s,dim_w,Bi_s);
[A_R2, B_R2, H2, one_R2, dim_R2, zer_R2] = ChooseRSW_new(A(length(A)/2+1:length(A)), B((length(p)/2+1:length(p)),length(A)/2+1:length(A)), p(length(p)/2+1:length(p)), K, dim_s,dim_w,Bi_s);

H = [H1,H2]; % this is the direction matrix
A_R = [A_R1,A_R2]; % ATE coefficient matrix
B_R = [B_R1,zeros(size(B_R2));zeros(size(B_R1)),B_R2]; % data restriction coefficient matrix
one_R = [one_R1,zeros(size(one_R2));zeros(size(one_R1)),one_R2]; % probability restriction coefficient matrix
zer_R = [zer_R1;zer_R2]; 
dim_R = dim_R1+dim_R2;

% use the direction matrix to create LATE target coefficient
for i=1:3
    for j=1:dim_x
        temp = LATE(j,:,i);
        LATE_R(j,:,i)= temp(H~=-1);
    end
end

% use direction matrix to create monotonicity and concavity restriction
% coefficient matrix
for i=1:size(M,1)
    temp = M(i,:);
    M_R(i,:) = temp(H~=-1);
end

for i=1:size(C,1)
    temp = C(i,:);
    C_R(i,:) = temp(H~=-1);
end

%% Determining U0 direction
% choose direction under assumption U0
G_M1 = ChooseU0(A(1:length(A)/2), B((1:length(p)/2),1:length(A)/2), p(1:length(p)/2), K, dim_s,dim_w,Bi_s);
G_M2 = ChooseU0(A(length(A)/2+1:length(A)), B((length(p)/2+1:length(p)),length(A)/2+1:length(A)), p(length(p)/2+1:length(p)), K, dim_s,dim_w,Bi_s);
G_M=[G_M1,zeros(size(G_M1,1),size(G_M1,2));zeros(size(G_M2,1),size(G_M2,2)),G_M2];

%% CVX - ATE
% linear programming: baseline case with W, no other assumptions
% create the base for the tuning parameter
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaIneq

minimize(norm(B*theta - p))
subject to
lambdaEq: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
cvx_end

base = cvx_optval;

% calculate the upper bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq

maximize( A * theta )
subject to
lambdaEq: norm(B * theta - p) <= base + 2*(1e-3); % included a tuning parameter to take sampling error into consideration
lambdaEq2: one * theta == ones(K*dim_x,1);
lambdaIneq: theta >= zer;
cvx_end

UB_ATE = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim)

dual variables lambdaEq lambdaEq2 lambdaIneq

minimize(A * theta)
subject to
lambdaEq: norm(B * theta - p) <= base + 2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer

cvx_end
LB_ATE = cvx_optval;

%% CVX - ATE & RSW
% linear programming with W, under assumption U*
% create the base for tuning parameter 
cvx_clear
cvx_begin
variable theta(dim_R_old)
dual variables lambdaEq2 lambdaIneq  

minimize( norm(B_R_old * theta - p) )
subject to
lambdaEq2: one_R_old * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R_old
cvx_end

base_R_old = cvx_optval;

% upper bound
cvx_clear
cvx_begin
variable theta(dim_R_old)
dual variables lambdaEq lambdaEq2 lambdaIneq

maximize( A_R_old * theta )
subject to
lambdaEq: norm(B_R_old * theta - p) <=  base_R_old + 2*(1e-3)
lambdaEq2: one_R_old * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R_old
cvx_end

UB_ATE_R_old = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim_R_old)
dual variables lambdaEq lambdaEq2 lambdaIneq

minimize(A_R_old * theta)
subject to
lambdaEq: norm(B_R_old * theta - p) <=  base_R_old + 2*(1e-3)
lambdaEq2: one_R_old * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R_old
cvx_end
LB_ATE_R_old = cvx_optval;

%% CVX - ATE WITH U0
% linear programming with W, under U0
% create base for tuning parameter
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq2 lambdaIneq lambdaIneq2

minimize( norm(B * theta-p) )
subject to
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: G_M*theta>=zeros(size(G_M,1),1)
cvx_end

base_0 = cvx_optval; 

% upper bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2

maximize( A * theta )
subject to

lambdaEq: norm(B * theta - p) <=base_0 + 2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: G_M*theta>=zeros(size(G_M,1),1)
cvx_end

UB_ATE0 = cvx_optval; 

% lower bound
cvx_clear
cvx_begin
variable theta(dim)

dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2

minimize(A * theta)
subject to
lambdaEq: norm(B * theta - p) <=base_0 + 2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: G_M*theta>=zeros(size(G_M,1),1)

cvx_end
LB_ATE0 = cvx_optval;
%% CVX - ATE & RSW new
% linear programming with W, under assumption U
% create base for tuning parameter
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq2 lambdaIneq  

minimize( norm(B_R * theta-p) )
subject to
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R
cvx_end

base_R = cvx_optval;

% upper bound
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq lambdaEq2 lambdaIneq 

maximize( A_R * theta )
subject to
lambdaEq: norm(B_R * theta - p) <=  base_R+2*(1e-3)
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R
cvx_end

UB_ATE_R = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq lambdaEq2 lambdaIneq

minimize(A_R * theta)
subject to
lambdaEq: norm(B_R * theta - p) <=  base_R+2*(1e-3)
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R
cvx_end
LB_ATE_R = cvx_optval;

%% CVX - ATE shape restriction
% linear programming with W, under assumption M and C
% create base for tuning
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq2 lambdaIneq lambdaIneq2 lambdaIneq3

minimize(norm(B*theta-p))
subject to
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: M * theta >= zeros(8*(K-1),1)
lambdaIneq3: C * theta <= zeros(8*(K-2),1)
cvx_end

base_S = cvx_optval;

% upper bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2 lambdaIneq3

maximize( A * theta )
subject to
lambdaEq: norm(B * theta - p)<=base_S+2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: M * theta >= zeros(8*(K-1),1)
lambdaIneq3: C * theta <= zeros(8*(K-2),1)
cvx_end

UB_ATE_S = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2 lambdaIneq3

minimize(A * theta)
subject to
lambdaEq: norm(B * theta - p)<=base_S+2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
lambdaIneq2: M * theta >= zeros(8*(K-1),1)
lambdaIneq3: C * theta <= zeros(8*(K-2),1)
cvx_end
LB_ATE_S = cvx_optval;

%% CVX - LATEs
% LP for LATEs under different assumptions
for i=1:2
    for j=1:3
        % baseline: upper bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        maximize( LATE(i,:,j) * theta )
        subject to
        lambdaEq: norm(B * theta - p)<=base+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        
        cvx_end
        UB_LATE(i,j) = cvx_optval;
        
        % baseline: lower bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        minimize(LATE(i,:,j) * theta)
        subject to
        lambdaEq: norm(B * theta - p)<=base+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        
        cvx_end
        LB_LATE(i,j) = cvx_optval;
        
        % assumption U0
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2
        
        maximize( LATE(i,:,j) * theta )
        subject to
        lambdaEq: norm(B * theta - p)<=base_0+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        lambdaIneq2: G_M*theta>=zeros(size(G_M,1),1)
        
        cvx_end
        UB_LATE0(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2
        
        minimize(LATE(i,:,j) * theta)
        subject to
        lambdaEq: norm(B * theta - p)<=base_0+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        lambdaIneq2: G_M*theta>=zeros(size(G_M,1),1)
        
        cvx_end
        LB_LATE0(i,j) = cvx_optval;
        
        % assumption U
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim_R)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        maximize( LATE_R(i,:,j) * theta )
        subject to
        lambdaEq: norm(B_R * theta - p)<=base_R+2*(1e-3)
        lambdaEq2: one_R * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R
        
        cvx_end
        UB_LATE_R(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim_R)
        
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        minimize(LATE_R(i,:,j) * theta)
        subject to
        
        lambdaEq: norm(B_R * theta - p)<=base_R+2*(1e-3)
        lambdaEq2: one_R * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R
        
        cvx_end
        LB_LATE_R(i,j) = cvx_optval;
        
        
        % assumption U*
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim_R_old)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        maximize( LATE_R_old(i,:,j) * theta )
        subject to
        
        lambdaEq: norm(B_R_old * theta - p)<= base_R_old+2*(1e-3)
        lambdaEq2: one_R_old * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R_old
        
        cvx_end
        UB_LATE_R_old(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim_R_old)
        
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        minimize(LATE_R_old(i,:,j) * theta)
        subject to
        
        lambdaEq: norm(B_R_old * theta - p)<= base_R_old+2*(1e-3)
        lambdaEq2: one_R_old * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R_old
        
        cvx_end
        LB_LATE_R_old(i,j) = cvx_optval;
       
        % assumption M and C
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2 lambdaIneq3
        
        maximize( LATE(i,:,j) * theta )
        subject to
        
        lambdaEq: norm(B * theta - p)<=base_S+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        lambdaIneq2: M * theta >= zeros(8*(K-1),1)
        lambdaIneq3: C * theta <= zeros(8*(K-2),1)
        
        cvx_end
        UB_LATE_S(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq lambdaIneq2 lambdaIneq3
        
        minimize(LATE(i,:,j) * theta)
        subject to
        lambdaEq: norm(B * theta - p)<=base_S+2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        lambdaIneq2: M * theta >= zeros(8*(K-1),1)
        lambdaIneq3: C * theta <= zeros(8*(K-2),1)
        
        cvx_end
        LB_LATE_S(i,j) = cvx_optval;
    end
end
%% Without W
%% Defining A, A is for ATE
dim_s = (2^dim_d); % number of maps

s = (1:dim_s)';
Bi_s = de2bi(s-1); % value of y(d)

% some probabilities used in calculating A
p_x = null(2,1);
for i=1:dim_x
    p_x(i) = sum((X==supp_x(i)))/N;
end

% Create target coefficient matrix A
A_0 = zeros(K,dim_s,dim_x);
A_1 = zeros(K,dim_s,dim_x);

gam0_ATE = zeros(K,1);
gam1_ATE = zeros(K,1);
for i=1:K
    syms u
    gam0_ATE(i) = -int(b{i},u,0,1);
    gam1_ATE(i) = int(b{i},u,0,1);
end

for i = 1:dim_x
    for j = 1:dim_s
        if Bi_s(j,1) == 1  % choose s (decimal value j) s.t. y(0) = 1
            A_0(:,j,i) = p_x(i);
        end
        if Bi_s(j,2) == 1  % choose s (decimal value j) s.t. y(1) = 1
            A_1(:,j,i) = p_x(i);
        end
        A_0(:,j,i) = A_0(:,j,i).*gam0_ATE;
        A_1(:,j,i) = A_1(:,j,i).*gam1_ATE;
    end
end

A = reshape(A_0,1,[])+reshape(A_1,1,[]);

%% Defining the restriction distributions p=P(D,Y=1|Z,X)
dim_p = dim_d*dim_z*dim_x;    % number of possibility in (Z,D,Y,X)
                              % there are some redundant probabilities
                              % ignored (r.f.t definition 3.1)
Bi = de2bi(0:dim_p-1);    % the order of column: (Z,D,Y,X); each ROW of this matrix correspond to each decimal number

% this is the data restriction
p = zeros(dim_p,1);
for i = 1:dim_p
    summand1 = (D==Bi(i,2)).*(X==Bi(i,3)).*(Z==Bi(i,1)).*(Y==1);
    summand2 = (Z==Bi(i,1)).*(X==Bi(i,3));
    p(i) = sum(summand1)/sum(summand2);
end

p_z = null(2,1); % propesntiy score, conditional on X
for j=1:dim_x
    for i=1:dim_z
        p_z(i,j) = sum((D==1).*(Z==supp_z(i)).*(X==supp_x(j)))/sum((Z==supp_z(i)).*(X==supp_x(j)));
    end
end

%% Defining coefficient matrices LATE for LATEs(NT, AT, complier)
gam0_LATE = zeros(K,dim_x,3);
gam1_LATE = zeros(K,dim_x,3);

for i=1:K
    for j=1:dim_x
        for m=1:3
            syms u
            gam0_LATE(i,j,m) = -int(b{i},u,(m==2)*p_z(1,j)+(m==3)*p_z(2,j),(m==1)*p_z(1,j)+(m==2)*p_z(2,j)+(m==3))/...
                ((m==1)*p_z(1,j)+(m==2)*p_z(2,j)+(m==3)-(m==2)*p_z(1,j)-(m==3)*p_z(2,j));
            gam1_LATE(i,j,m) = int(b{i},u,(m==2)*p_z(1,j)+(m==3)*p_z(2,j),(m==1)*p_z(1,j)+(m==2)*p_z(2,j)+(m==3))/...
                ((m==1)*p_z(1,j)+(m==2)*p_z(2,j)+(m==3)-(m==2)*p_z(1,j)-(m==3)*p_z(2,j));
        end
    end
end

LATE = zeros(2,K*dim_s*dim_x,3);
for m=1:3
    L_0_0 = zeros(K,dim_s,dim_x);
    L_1_0 = zeros(K,dim_s,dim_x);
    L_0_1 = zeros(K,dim_s,dim_x);
    L_1_1 = zeros(K,dim_s,dim_x);
    for i = 1:dim_x
        for j = 1:dim_s
            if Bi_s(j,1) == 1  % 
                L_0_0(:,j,i) = -1*i+2;
                L_0_1(:,j,i) = i-1;
            end
            if Bi_s(j,2) == 1  % 
                L_1_0(:,j,i) = -1*i+2;
                L_1_1(:,j,i) = i-1;
            end
            L_0_0(:,j,i) = L_0_0(:,j,i).*gam0_LATE(:,i,m);
            L_1_0(:,j,i) = L_1_0(:,j,i).*gam1_LATE(:,i,m);
            L_0_1(:,j,i) = L_0_1(:,j,i).*gam0_LATE(:,i,m);
            L_1_1(:,j,i) = L_1_1(:,j,i).*gam1_LATE(:,i,m);
        end
    end
    LATE(1,:,m)=reshape(L_0_0,1,[])+reshape(L_1_0,1,[]);
    LATE(2,:,m)=reshape(L_0_1,1,[])+reshape(L_1_1,1,[]);
end

%% Defining B
B = [];
for l = 1:dim_p
    % focus on p(l) and find corresponding (d,y,z) in p(d,y|z)
    d = Bi(l,2);
    y = 1;
    z = Bi(l,1);
    x = Bi(l,3);
    delta = null(1,K);
    for m=1:K
        if d == 1
            delta(m) = int(b{m},u,0,p_z(z+1,x+1));
        elseif d == 0
            delta(m) = int(b{m},u,p_z(z+1,x+1),1);
        end
    end
    
    B0 = zeros(K,dim_s,dim_x);
    
    for j = 1:dim_s
        if Bi_s(j,bi2de(d)+1) == y
            B0(:,j,x+1) = 1;
        end
        B0(:,j,x+1) = (B0(:,j,x+1)'.*delta)';
    end
    B0 = reshape(B0,1,[]);
    B(l,:) = B0;
end

%% Restrictions on conditional probabilities
% The one without rank similarity assn
dim = size(B,2);
zer = zeros(dim,1);

one = zeros(K,dim,dim_x);

for i=1:K
    for j=1:dim
        if mod(j-i,K)==0 && j<=dim_s*K
            one(i,j,1) = 1;
        elseif mod(j-i,K)==0 && j>dim_s*K
            one(i,j,2) = 1;
        end
    end
end

one = [one(:,:,1);one(:,:,2)];

%% Defining additional assumptions
% Monotonicity (assumption M)
M0 = zeros(dim_x*(K-1),dim);
M1 = zeros(dim_x*(K-1),dim);

for x = 1:dim_x
    if x==1
        for i = 1:K-1
            M_0 = zeros(K,dim_s);
            M_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % choose s (decimal value j) s.t. y(0) = 1
                    M_0(i,j) = -1;
                    M_0(i+1,j) = 1;
                end
                if Bi_s(j,2) == 1 % choose s (decimal value j) s.t. y(1) = 1
                    M_1(i,j) = -1;
                    M_1(i+1,j) = 1;
                end
            end
            M_0 = reshape(M_0,1,[]);
            M_1 = reshape(M_1,1,[]);
            M0(i,1:dim/2) = M_0;
            M1(i,1:dim/2) = M_1;
        end
    else
        for i = K:2*(K-1)
            M_0 = zeros(K,dim_s);
            M_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % choose s (decimal value j) s.t. y(0) = 1
                    M_0(i-K+1,j) = -1;
                    M_0(i-K+2,j) = 1;
                end
                if Bi_s(j,2) == 1 % choose s (decimal value j) s.t. y(1) = 1
                    M_1(i-K+1,j) = -1;
                    M_1(i-K+2,j) = 1;
                end
            end
            M_0 = reshape(M_0,1,[]);
            M_1 = reshape(M_1,1,[]);
            M0(i,dim/2+1:dim) = M_0;
            M1(i,dim/2+1:dim) = M_1;
        end
    end
end

M = [M0;M1];

% Concavity
C0 = zeros(dim_x*(K-2),dim);
C1 = zeros(dim_x*(K-2),dim);
for x = 1:dim_x
    if x==1
        for i = 1:K-2
            C_0 = zeros(K,dim_s);
            C_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % choose s (decimal value j) s.t. y(0) = 1
                    C_0(i,j) = 1;
                    C_0(i+1,j) = -2;
                    C_0(i+2,j) = 1;
                end
                if Bi_s(j,2) == 1 % choose s (decimal value j) s.t. y(1) = 1
                    C_1(i,j) = 1;
                    C_1(i+1,j) = -2;
                    C_1(i+2,j) = 1;
                end
            end
            C_0 = reshape(C_0,1,[]);
            C0(i,1:dim/2) = C_0;
            C_1 = reshape(C_1,1,[]);
            C1(i,1:dim/2) = C_1;
        end
    else
        for i = K-1:dim_x*(K-2)
            C_0 = zeros(K,dim_s);
            C_1 = zeros(K,dim_s);
            for j=1:dim_s
                if Bi_s(j,1) == 1 % choose s (decimal value j) s.t. y(0) = 1
                    C_0(i-K+2,j) = 1;
                    C_0(i-K+3,j) = -2;
                    C_0(i-K+4,j) = 1;
                end
                if Bi_s(j,2) == 1 % choose s (decimal value j) s.t. y(1) = 1
                    C_1(i-K+2,j) = 1;
                    C_1(i-K+3,j) = -2;
                    C_1(i-K+4,j) = 1;
                end
            end
            C_0 = reshape(C_0,1,[]);
            C0(i,dim/2+1:dim) = C_0;
            C_1 = reshape(C_1,1,[]);
            C1(i,dim/2+1:dim) = C_1;
        end
    end
end

C = [C0;C1];
%% Determining the RS direction
% choose direction under assumption U0
% choose direction for x=0
[A_R1, B_R1, H1, one_R1, dim_R1, zer_R1] = ChooseRS(A(1:length(A)/2), B((1:length(p)/2),1:length(A)/2), p(1:length(p)/2), K);
% choose direction for x=1
[A_R2, B_R2, H2, one_R2, dim_R2, zer_R2] = ChooseRS(A(length(A)/2+1:length(A)), B((length(p)/2+1:length(p)),length(A)/2+1:length(A)), p(length(p)/2+1:length(p)), K);

H = [H1,H2]; % the direction matrix
A_R = [A_R1,A_R2]; % the target coefficient matrix
B_R = [B_R1,zeros(size(B_R2));zeros(size(B_R1)),B_R2]; % the data restriction matrix
one_R = [one_R1,zeros(size(one_R2));zeros(size(one_R1)),one_R2]; % the probability restriction matrix
zer_R = [zer_R1;zer_R2];
dim_R = dim_R1+dim_R2;

% create LATE coefficient matrix
LATE_R=[];
for i=1:3
    for j=1:dim_x
        temp = LATE(j,:,i);
        LATE_R(j,:,i)= temp(H~=-1);
    end
end

% create matrices for assumption M and C under assumption U
M_R=[];
C_R=[];
for i=1:size(M,1)
    temp = M(i,:);
    M_R(i,:) = temp(H~=-1);
end

for i=1:size(C,1)
    temp = C(i,:);
    C_R(i,:) = temp(H~=-1);
end

%% CVX - ATE
% baseline result, no additional assumption
% base for tuning parameter
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq2 lambdaIneq

minimize( norm(B*theta-p) )
subject to
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
cvx_end

base_ATE3 = cvx_optval;

% upper bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq

maximize( A * theta )
subject to
lambdaEq: norm(B * theta - p) <=  base_ATE3 + 2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer
cvx_end

UB_ATE3 = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim)
dual variables lambdaEq lambdaEq2 lambdaIneq

minimize(A * theta)
subject to
lambdaEq: norm(B * theta - p) <=  base_ATE3 + 2*(1e-3)
lambdaEq2: one * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer

cvx_end
LB_ATE3 = cvx_optval;

%% CVX - ATE RS
% linear programming under assumption U
% base for tuning
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq2 lambdaIneq

minimize( norm(B_R*theta-p) )
subject to
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R
cvx_end

base_ATE_R3 = cvx_optval;

% upper bound
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq lambdaEq2 lambdaIneq

maximize( A_R * theta )
subject to
lambdaEq: norm(B_R * theta - p) <= base_ATE_R3+2*(1e-3)
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R
cvx_end

UB_ATE_R3 = cvx_optval;

% lower bound
cvx_clear
cvx_begin
variable theta(dim_R)
dual variables lambdaEq lambdaEq2 lambdaIneq

minimize(A_R * theta)
subject to
lambdaEq: norm(B_R * theta - p) <= base_ATE_R3+2*(1e-3)
lambdaEq2: one_R * theta == ones(K*dim_x,1)
lambdaIneq: theta >= zer_R

cvx_end
LB_ATE_R3 = cvx_optval;

%% CVX - LATEs
for i=1:2
    for j=1:3
        % baseline results
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq
       
        maximize( LATE(i,:,j) * theta )
        subject to
        lambdaEq: norm(B * theta - p) <=  base_ATE3 + 2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        cvx_end
        
        UB_LATE3(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        minimize(LATE(i,:,j) * theta)
        subject to
        lambdaEq: norm(B * theta - p) <=  base_ATE3 + 2*(1e-3)
        lambdaEq2: one * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer
        
        cvx_end
        LB_LATE3(i,j) = cvx_optval;
        
        % assumption U
        % upper bound
        cvx_clear
        cvx_begin
        variable theta(dim_R)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        maximize( LATE_R(i,:,j) * theta )
        subject to
        lambdaEq: norm(B_R * theta - p) <= base_ATE_R3+2*(1e-3)
        lambdaEq2: one_R * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R
        cvx_end
        
        UB_LATE_R3(i,j) = cvx_optval;
        
        % lower bound
        cvx_clear
        cvx_begin
        variable theta(dim_R)
        dual variables lambdaEq lambdaEq2 lambdaIneq
        
        minimize(LATE_R(i,:,j) * theta)
        subject to
        lambdaEq: norm(B_R * theta - p) <= base_ATE_R3+2*(1e-3)
        lambdaEq2: one_R * theta == ones(K*dim_x,1)
        lambdaIneq: theta >= zer_R
        
        cvx_end
        LB_LATE_R3(i,j) = cvx_optval;
    end
end

%% Plot for ATE: 
x = 0.01:0.01:0.99;
x1 = LB_ATE3*ones(1,99); % No assn, without W LB
x2 = UB_ATE3*ones(1,99); % No assn, without W UB
x3 = LB_ATE_R3*ones(1,99); % U0, without W LB
x4 = UB_ATE_R3*ones(1,99); % U0, without W UB
y1 = LB_ATE*ones(1,99); % with W LB
y2 = UB_ATE*ones(1,99); % with W UB
y3 = LB_ATE_R*ones(1,99); % RS+W LB
y4 = UB_ATE_R*ones(1,99); % RS+W UB
y5 = LB_ATE_S*ones(1,99); % M+C+W LB
y6 = UB_ATE_S*ones(1,99); % M+C+W UB
y7 = LB_ATE0*ones(1,99); % U0 LB
y8 = UB_ATE0*ones(1,99); % U0 UB
y9 = LB_ATE_R_old*ones(1,99); % RS old+W LB
y10 = UB_ATE_R_old*ones(1,99); % RS old+W UB

% The population level ATE
y = double((1-p_W)*(1-p_X)*(0.02*int(bern(4,0,u),u,0,1)+0.04*int(bern(4,1,u),u,0,1)+0.07*int(bern(4,2,u),u,0,1)+0.1*int(bern(4,3,u),u,0,1)+0.1*int(bern(4,4,u),u,0,1))+...
    p_W*(1-p_X)*(0.89*int(bern(4,0,u),u,0,1)+0.41*int(bern(4,1,u),u,0,1)+0.23*int(bern(4,2,u),u,0,1)+0.12*int(bern(4,3,u),u,0,1)+0.1*int(bern(4,4,u),u,0,1))+...
    (1-p_W)*p_X*(0*int(bern(4,0,u),u,0,1)+0.03*int(bern(4,1,u),u,0,1)+0.06*int(bern(4,2,u),u,0,1)+0.09*int(bern(4,3,u),u,0,1)+0.09*int(bern(4,4,u),u,0,1))+...
    p_W*p_X*(0.69*int(bern(4,0,u),u,0,1)+0.35*int(bern(4,1,u),u,0,1)+0.13*int(bern(4,2,u),u,0,1)+0.05*int(bern(4,3,u),u,0,1)+0.05*int(bern(4,4,u),u,0,1)))*ones(1,99);

figure('Name','ATE: first figure');
plot(x,y,'r','linew',2)
hold on
% No assn, without W
plot (x,x1,'-o','color',[0 1 1]*1,'linew',1.5,'MarkerIndices',1:5:100) %LB
plot (x,x2,'-o','color',[0 1 1]*1,'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100) %UB
% with W
plot (x,y1,':','color',[0 1 1]*0.4,'linew',2) %LB
plot (x,y2,':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off') %UB
% U0, without W
plot (x,x3,'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)%LB
plot (x,x4,'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)%UB
% U0(or U, same result)+W
plot (x,y7,'--','color',[0 1 0],'linew',1.5)%LB
plot (x,y8,'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')%UB
% M+C+W
plot (x,y5,'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100) %LB
plot (x,y6,'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)%UB

hold off
legend('True ATE', 'Bounds on ATE: No Assumption','Bounds on ATE: W','ATE: U0','Bounds on ATE: W+U0','Bounds on ATE: W+M+C','Location','bestoutside','orientation','horizontal','Fontsize',11)
title('Bounds on ATE')
xlabel('u \in [0,1]')  
lgd = legend;
lgd.NumColumns = 2;
set(0,'defaultfigurecolor','w') 

%% Plot for ATE: second figure
figure('Name','ATE: second figure');
plot(x,y,'r','linew',2)
hold on
% No assn, without W
plot (x,x1,'-o','color',[0 1 1]*1,'linew',1.5,'MarkerIndices',1:5:100) %LB
plot (x,x2,'-o','color',[0 1 1]*1,'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100) %UB
% U0+W
plot (x,y7,'--','color',[0 1 0],'linew',1.5)%LB
plot (x,y8,'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')%UB
% U* + W
plot (x,y9,'-.','color',[0 0.4 0.6],'linew',1)%LB
plot (x,y10,'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')%UB

hold off
legend('True ATE', 'Bounds on ATE: No Assumption','Bounds on ATE: W+U0','Bounds on ATE: W+U*','Location','bestoutside','orientation','horizontal','Fontsize',11)
title('Bounds on ATE')
xlabel('u \in [0,1]')  
lgd = legend;
lgd.NumColumns = 2;
set(0,'defaultfigurecolor','w') 

%% Plot for LATE: first figure
syms u
% true always taker LATE z1:X=0, z2:X=1
z1 = double((0.02*(1-p_W)+0.89*p_W)*int(bern(4,0,u),u,0,0.1)+...
    (0.04*(1-p_W)+0.41*p_W)*int(bern(4,1,u),u,0,0.1)+...
    (0.07*(1-p_W)+0.23*p_W)*int(bern(4,2,u),u,0,0.1)+...
    (0.1*(1-p_W)+0.12*p_W)*int(bern(4,3,u),u,0,0.1)+...
    (0.1*(1-p_W)+0.1*p_W)*int(bern(4,4,u),u,0,0.1))/0.1;
z2 = double((0*(1-p_W)+0.69*p_W)*int(bern(4,0,u),u,0,0.4)+...
    (0.03*(1-p_W)+0.35*p_W)*int(bern(4,1,u),u,0,0.4)+...
    (0.06*(1-p_W)+0.13*p_W)*int(bern(4,2,u),u,0,0.4)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,3,u),u,0,0.4)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,4,u),u,0,0.4))/0.4;
% true complier LATE z3:X=0, z4:X=1
z3 = double((0.02*(1-p_W)+0.89*p_W)*int(bern(4,0,u),u,0.1,0.4)+...
    (0.04*(1-p_W)+0.41*p_W)*int(bern(4,1,u),u,0.1,0.4)+...
    (0.07*(1-p_W)+0.23*p_W)*int(bern(4,2,u),u,0.1,0.4)+...
    (0.1*(1-p_W)+0.12*p_W)*int(bern(4,3,u),u,0.1,0.4)+...
    (0.1*(1-p_W)+0.1*p_W)*int(bern(4,4,u),u,0.1,0.4))/0.3;
z4 = double((0*(1-p_W)+0.69*p_W)*int(bern(4,0,u),u,0.4,0.7)+...
    (0.03*(1-p_W)+0.35*p_W)*int(bern(4,1,u),u,0.4,0.7)+...
    (0.06*(1-p_W)+0.13*p_W)*int(bern(4,2,u),u,0.4,0.7)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,3,u),u,0.4,0.7)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,4,u),u,0.4,0.7))/0.3;
% true never taker LATE z5:X=0, z6:X=1
z5 = double((0.02*(1-p_W)+0.89*p_W)*int(bern(4,0,u),u,0.4,1)+...
    (0.04*(1-p_W)+0.41*p_W)*int(bern(4,1,u),u,0.4,1)+...
    (0.07*(1-p_W)+0.23*p_W)*int(bern(4,2,u),u,0.4,1)+...
    (0.1*(1-p_W)+0.12*p_W)*int(bern(4,3,u),u,0.4,1)+...
    (0.1*(1-p_W)+0.1*p_W)*int(bern(4,4,u),u,0.4,1))/0.6;
z6 = double((0*(1-p_W)+0.69*p_W)*int(bern(4,0,u),u,0.7,1)+...
    (0.03*(1-p_W)+0.35*p_W)*int(bern(4,1,u),u,0.7,1)+...
    (0.06*(1-p_W)+0.13*p_W)*int(bern(4,2,u),u,0.7,1)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,3,u),u,0.7,1)+...
    (0.09*(1-p_W)+0.05*p_W)*int(bern(4,4,u),u,0.7,1))/0.3;

% true MTE: X=0
mte_0 = (0.02*(1-p_W)+0.89*p_W)*bern(4,0,x) + ...
    (0.04*(1-p_W)+0.41*p_W)*bern(4,1,x) + ...
    (0.07*(1-p_W)+0.23*p_W)*bern(4,2,x) + ...
    (0.1*(1-p_W)+0.12*p_W)*bern(4,3,x) +...
    (0.1*(1-p_W)+0.1*p_W)*bern(4,4,x);  

% true MTE: X=1
mte_1 = (0*(1-p_W)+0.69*p_W)*bern(4,0,x) + ...
    (0.03*(1-p_W)+0.35*p_W)*bern(4,1,x) + ...
    (0.06*(1-p_W)+0.13*p_W)*bern(4,2,x) + ...
    (0.09*(1-p_W)+0.05*p_W)*bern(4,3,x) + ...
    (0.09*(1-p_W)+0.05*p_W)*bern(4,4,x);


% LATEs: X=0
subplot(1,2,1)
plot(x,mte_0,'r','linew',2)
hold on
line([0,0],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.1,0.1],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.4,0.4],[-1 -0.85],'Color','r','HandleVisibility','off')
line([1,1],[-1 -0.85],'Color','r','HandleVisibility','off')

line([0,0.008],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.092,0.1],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.1,0.198],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.28,0.4],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.4,0.63],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.75,1],[-0.9,-0.9],'Color','r','HandleVisibility','off')

xline(0.1,'r:','HandleVisibility','off');
xline(0.4,'r:','HandleVisibility','off');

% AT-LATE: X=0
% No assn, without W
plot (0.01:0.01:0.1,UB_LATE3(1,1)*ones(1,10),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.1,LB_LATE3(1,1)*ones(1,10),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% with W
plot (0.01:0.01:0.1,UB_LATE(1,1)*ones(1,10),':','color',[0 1 1]*0.4,'linew',2)
plot (0.01:0.01:0.1,LB_LATE(1,1)*ones(1,10),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, witout W
plot (0.01:0.01:0.1,UB_LATE_R3(1,1)*ones(1,10),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.1,LB_LATE_R3(1,1)*ones(1,10),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0 + W
plot (0.01:0.01:0.1,UB_LATE0(1,1)*ones(1,10),'--','color',[0 1 0],'linew',1.5)
plot (0.01:0.01:0.1,LB_LATE0(1,1)*ones(1,10),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.01:0.01:0.1,UB_LATE_S(1,1)*ones(1,10),'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.1,LB_LATE_S(1,1)*ones(1,10),'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% C-LATE: X=0
% No assn, without W
plot (0.11:0.01:0.4,UB_LATE3(1,2)*ones(1,30),'-o','color',[0 1 1]*1,'linew',1.5,'MarkerIndices',1:5:100)
plot (0.11:0.01:0.4,LB_LATE3(1,2)*ones(1,30),'-o','color',[0 1 1]*1,'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% with W
plot (0.11:0.01:0.4,UB_LATE(1,2)*ones(1,30),':','color',[0 1 1]*0.4,'linew',2)
plot (0.11:0.01:0.4,LB_LATE(1,2)*ones(1,30),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, without W
plot (0.11:0.01:0.4,UB_LATE_R3(1,2)*ones(1,30),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.11:0.01:0.4,LB_LATE_R3(1,2)*ones(1,30),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.11:0.01:0.4,UB_LATE0(1,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5)
plot (0.11:0.01:0.4,LB_LATE0(1,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.11:0.01:0.4,UB_LATE_S(1,2)*ones(1,30),'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.11:0.01:0.4,LB_LATE_S(1,2)*ones(1,30),'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% NT-LATE: X=0
% No assn, without W
plot (0.41:0.01:0.99,UB_LATE3(1,3)*ones(1,59),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.99,LB_LATE3(1,3)*ones(1,59),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% with W
plot (0.41:0.01:0.99,UB_LATE(1,3)*ones(1,59),':','color',[0 1 1]*0.4,'linew',2)
plot (0.41:0.01:0.99,LB_LATE(1,3)*ones(1,59),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, without W
plot (0.41:0.01:0.99,UB_LATE_R3(1,3)*ones(1,59),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.99,LB_LATE_R3(1,3)*ones(1,59),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.41:0.01:0.99,UB_LATE0(1,3)*ones(1,59),'--','color',[0 1 0],'linew',1.5)
plot (0.41:0.01:0.99,LB_LATE0(1,3)*ones(1,59),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.41:0.01:0.99,UB_LATE_S(1,3)*ones(1,59),'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.99,LB_LATE_S(1,3)*ones(1,59),'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% TEXTS
ylim([-1 1])
txt1 = {'LATE-AT'};
text(0.005,-0.9,txt1,'Fontsize',5.5)
txt2 = {'LATE-C'};
text(0.2,-0.9,txt2,'Fontsize',5.5)
txt3 = {'LATE-NT'};
text(0.65,-0.9,txt3,'Fontsize',5.5)
hold off
xlabel('u \in [0,1]')
title('Bounds on LATEs: X=0')

% LATEs: X=1
subplot(1,2,2)
plot(x,mte_1,'r','linew',3)
hold on
line([0,0],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.4,0.4],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.7,0.7],[-1 -0.85],'Color','r','HandleVisibility','off')
line([1,1],[-1 -0.85],'Color','r','HandleVisibility','off')

line([0,0.13],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.25,0.4],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.4,0.48],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.59,0.7],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.7,0.78],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.9,1],[-0.9,-0.9],'Color','r','HandleVisibility','off')

xline(0.4,'r:','HandleVisibility','off');
xline(0.7,'r:','HandleVisibility','off');

% AT-LATE: X=1
% No assn, without W
plot (0.01:0.01:0.4,UB_LATE3(2,1)*ones(1,40),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.4,LB_LATE3(2,1)*ones(1,40),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% with W
plot (0.01:0.01:0.4,UB_LATE(2,1)*ones(1,40),':','color',[0 1 1]*0.4,'linew',2)
plot (0.01:0.01:0.4,LB_LATE(2,1)*ones(1,40),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, without W
plot (0.01:0.01:0.4,UB_LATE_R3(2,1)*ones(1,40),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.4,LB_LATE_R3(2,1)*ones(1,40),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.01:0.01:0.4,UB_LATE_R(2,1)*ones(1,40),'--','color',[0 1 0],'linew',1.5)
plot (0.01:0.01:0.4,LB_LATE_R(2,1)*ones(1,40),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.01:0.01:0.4,UB_LATE_S(2,1)*ones(1,40),'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.4,LB_LATE_S(2,1)*ones(1,40),'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% C-LATE: X=1
% No assn, without W
plot (0.41:0.01:0.7,UB_LATE3(2,2)*ones(1,30),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.7,LB_LATE3(2,2)*ones(1,30),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% wiht W
plot (0.41:0.01:0.7,UB_LATE(2,2)*ones(1,30),':','color',[0 1 1]*0.4,'linew',2)
plot (0.41:0.01:0.7,LB_LATE(2,2)*ones(1,30),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, without W
plot (0.41:0.01:0.7,UB_LATE_R3(2,2)*ones(1,30),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.7,LB_LATE_R3(2,2)*ones(1,30),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.41:0.01:0.7,UB_LATE0(2,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5)
plot (0.41:0.01:0.7,LB_LATE0(2,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.41:0.01:0.7,UB_LATE_S(2,2)*ones(1,30),'-x','color',[255/255 0.5 255/255],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.7,LB_LATE_S(2,2)*ones(1,30),'-x','color',[255/255 0.5 255/255],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% NT-LATE: X=1
% No assn, without W
plot (0.71:0.01:0.99,UB_LATE3(2,3)*ones(1,29),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.71:0.01:0.99,LB_LATE3(2,3)*ones(1,29),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% with W
plot (0.71:0.01:0.99,UB_LATE(2,3)*ones(1,29),':','color',[0 1 1]*0.4,'linew',2)
plot (0.71:0.01:0.99,LB_LATE(2,3)*ones(1,29),':','color',[0 1 1]*0.4,'linew',2,'HandleVisibility','off')
% U0, without W
plot (0.71:0.01:0.99,UB_LATE_R3(2,3)*ones(1,29),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.71:0.01:0.99,LB_LATE_R3(2,3)*ones(1,29),'-+','color',[0.4940 0.1840 0.5560],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.71:0.01:0.99,UB_LATE0(2,3)*ones(1,29),'--','color',[0 1 0],'linew',1.5)
plot (0.71:0.01:0.99,LB_LATE0(2,3)*ones(1,29),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% M+C+W
plot (0.71:0.01:0.99,UB_LATE_S(2,3)*ones(1,29),'-x','color',[1 0.5 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.71:0.01:0.99,LB_LATE_S(2,3)*ones(1,29),'-x','color',[1 0.5 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)

% TEXTS
ylim([-1 1])
txt1 = {'LATE-AT'};
text(0.15,-0.9,txt1,'Fontsize',5.5)
txt2 = {'LATE-C'};
text(0.5,-0.9,txt2,'Fontsize',5.5)
txt3 = {'LATE-NT'};
text(0.8,-0.9,txt3,'Fontsize',5.5)
hold off
xlabel('u \in [0,1]')  

title('Bounds on LATEs: X=1')
legend('DGP MTE', 'Bounds on LATEs: No Assumption','Bounds on LATEs: W','Bounds on LATEs: U0','Bounds on LATEs: W+U0','Bounds on LATEs: W+M+C','Location','eastoutside','orientation','horizontal','Fontsize',10) 
lgd = legend;
%lgd.Layout.Tile = 'eastoutside';
lgd.NumColumns = 4;
lgd.Position(1) = 0;
lgd.Position(2) = 0;
lgd.Position(3) = 0.1;
lgd.Position(4) = 0.1;
set(0,'defaultfigurecolor','w') 

%% Plot for LATE: second figure
% LATEs: X=0
subplot(1,2,1)
plot(x,mte_0,'r','linew',2)
hold on
line([0,0],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.1,0.1],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.4,0.4],[-1 -0.85],'Color','r','HandleVisibility','off')
line([1,1],[-1 -0.85],'Color','r','HandleVisibility','off')

line([0,0.008],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.092,0.1],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.1,0.198],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.28,0.4],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.4,0.63],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.75,1],[-0.9,-0.9],'Color','r','HandleVisibility','off')

xline(0.1,'r:','HandleVisibility','off');
xline(0.4,'r:','HandleVisibility','off');

% AT-LATE: X=0
% No assn, without W
plot (0.01:0.01:0.1,UB_LATE3(1,1)*ones(1,10),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.1,LB_LATE3(1,1)*ones(1,10),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0 + W
plot (0.01:0.01:0.1,UB_LATE_R(1,1)*ones(1,10),'--','color',[0 1 0],'linew',1.5)
plot (0.01:0.01:0.1,LB_LATE_R(1,1)*ones(1,10),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U* + W
plot (0.01:0.01:0.1,UB_LATE_R_old(1,1)*ones(1,10),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.01:0.01:0.1,LB_LATE_R_old(1,1)*ones(1,10),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% C-LATE: X=0
% No assn, without W
plot (0.11:0.01:0.4,UB_LATE3(1,2)*ones(1,30),'-o','color',[0 1 1]*1,'linew',1.5,'MarkerIndices',1:5:100)
plot (0.11:0.01:0.4,LB_LATE3(1,2)*ones(1,30),'-o','color',[0 1 1]*1,'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.11:0.01:0.4,UB_LATE0(1,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5)
plot (0.11:0.01:0.4,LB_LATE0(1,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U*+W
plot (0.11:0.01:0.4,UB_LATE_R_old(1,2)*ones(1,30),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.11:0.01:0.4,LB_LATE_R_old(1,2)*ones(1,30),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% NT-LATE: X=0
% No assn, without W
plot (0.41:0.01:0.99,UB_LATE3(1,3)*ones(1,59),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.99,LB_LATE3(1,3)*ones(1,59),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.41:0.01:0.99,UB_LATE0(1,3)*ones(1,59),'--','color',[0 1 0],'linew',1.5)
plot (0.41:0.01:0.99,LB_LATE0(1,3)*ones(1,59),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U*+W
plot (0.41:0.01:0.99,UB_LATE_R_old(1,3)*ones(1,59),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.41:0.01:0.99,LB_LATE_R_old(1,3)*ones(1,59),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% TEXTS
ylim([-1 1])
txt1 = {'LATE-AT'};
text(0.005,-0.9,txt1,'Fontsize',5.5)
txt2 = {'LATE-C'};
text(0.2,-0.9,txt2,'Fontsize',5.5)
txt3 = {'LATE-NT'};
text(0.65,-0.9,txt3,'Fontsize',5.5)
hold off
xlabel('u \in [0,1]')  
title('Bounds on LATEs: X=0')

% LATEs: X=1
subplot(1,2,2)
plot(x,mte_1,'r','linew',2)
hold on
line([0,0],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.4,0.4],[-1 -0.85],'Color','r','HandleVisibility','off')
line([0.7,0.7],[-1 -0.85],'Color','r','HandleVisibility','off')
line([1,1],[-1 -0.85],'Color','r','HandleVisibility','off')

line([0,0.13],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.25,0.4],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.4,0.48],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.59,0.7],[-0.9,-0.9],'Color','r','HandleVisibility','off')
line([0.7,0.78],[-0.9 -0.9],'Color','r','HandleVisibility','off')
line([0.9,1],[-0.9,-0.9],'Color','r','HandleVisibility','off')

xline(0.4,'r:','HandleVisibility','off');
xline(0.7,'r:','HandleVisibility','off');

% AT-LATE: X=1
% No assn, without W
plot (0.01:0.01:0.4,UB_LATE3(2,1)*ones(1,40),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.01:0.01:0.4,LB_LATE3(2,1)*ones(1,40),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.01:0.01:0.4,UB_LATE0(2,1)*ones(1,40),'--','color',[0 1 0],'linew',1.5)
plot (0.01:0.01:0.4,LB_LATE0(2,1)*ones(1,40),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U*+W
plot (0.01:0.01:0.4,UB_LATE_R_old(2,1)*ones(1,40),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.01:0.01:0.4,LB_LATE_R_old(2,1)*ones(1,40),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% C-LATE: X=1
% No assn, without W
plot (0.41:0.01:0.7,UB_LATE3(2,2)*ones(1,30),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.41:0.01:0.7,LB_LATE3(2,2)*ones(1,30),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.41:0.01:0.7,UB_LATE0(2,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5)
plot (0.41:0.01:0.7,LB_LATE0(2,2)*ones(1,30),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U*+W
plot (0.41:0.01:0.7,UB_LATE_R_old(2,2)*ones(1,30),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.41:0.01:0.7,LB_LATE_R_old(2,2)*ones(1,30),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% NT-LATE: X=1
% No assn, without W
plot (0.71:0.01:0.99,UB_LATE3(2,3)*ones(1,29),'-o','color',[0 1 1],'linew',1.5,'MarkerIndices',1:5:100)
plot (0.71:0.01:0.99,LB_LATE3(2,3)*ones(1,29),'-o','color',[0 1 1],'linew',1.5,'HandleVisibility','off','MarkerIndices',1:5:100)
% U0+W
plot (0.71:0.01:0.99,UB_LATE0(2,3)*ones(1,29),'--','color',[0 1 0],'linew',1.5)
plot (0.71:0.01:0.99,LB_LATE0(2,3)*ones(1,29),'--','color',[0 1 0],'linew',1.5,'HandleVisibility','off')
% U*+W
plot (0.71:0.01:0.99,UB_LATE_R_old(2,3)*ones(1,29),'-.','color',[0 0.4 0.6],'linew',1)
plot (0.71:0.01:0.99,LB_LATE_R_old(2,3)*ones(1,29),'-.','color',[0 0.4 0.6],'linew',1,'HandleVisibility','off')

% TEXTS
ylim([-1 1])
txt1 = {'LATE-AT'};
text(0.15,-0.9,txt1,'Fontsize',5.5)
txt2 = {'LATE-C'};
text(0.5,-0.9,txt2,'Fontsize',5.5)
txt3 = {'LATE-NT'};
text(0.8,-0.9,txt3,'Fontsize',5.5)
hold off
xlabel('u \in [0,1]')  
title('Bounds on LATEs: X=1')
legend('DGP MTE', 'Bounds on LATEs: No Assumption','Bounds on LATEs: W+U0','Bounds on LATEs: W+U*','Location','bestoutside','orientation','horizontal','Fontsize',10)
lgd = legend;
lgd.NumColumns = 4;
lgd.Position(1) = 0;
lgd.Position(2) = 0;
lgd.Position(3) = 0.1;
lgd.Position(4) = 0.1;
set(0,'defaultfigurecolor','w') 
